﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using HeuristicLab.Problems.BinPacking.Interfaces;
using HeuristicLab.Problems.BinPacking.PackingItem;
using HeuristicLab.Problems.BinPacking.Shapes;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Core;
using HeuristicLab.Common;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.BinPacking.PackingPlans {
  //RegularPackingPlan defines that the packingItems have to be parallel to the packingBin in every moment
  //SimpleRotation defines that the items may be rotated 
  //  only in one dimension, 
  //  only once and 
  //  only by 90degree
  //    so that parallelism to the correlating bin is ensured
  //This definition implies that the floor of the (3D) objects never changes (what may be a wanted constraint in most cases..)
  [Item("RegularSimpleRotationPackingPlan", "Represents a concrete solution for a bin-packing problem using simple rotation (90degrees-steps) and regular shapes (e.g. rectangle)")]
  [StorableClass]
  public class RegularSimpleRotationPackingPlan<D, B, I> : PackingPlan<D, B, I>
    where D : class, IPackingDimensions
    where B : PackingShape<D>, IRegularPackingShape, IPackingBin
    where I : PackingShape<D>, IRegularPackingShape, IPackingItem {
    
    #region Properties
    public ItemList<BoolValue> PackingItemsRotated { get; set; }
    #endregion

    public RegularSimpleRotationPackingPlan(B binMeasures, ItemList<I> itemMeasures) : base(binMeasures, itemMeasures) {

    }

    [StorableConstructor]
    protected RegularSimpleRotationPackingPlan(bool deserializing) : base(deserializing) { }
    protected RegularSimpleRotationPackingPlan(RegularSimpleRotationPackingPlan<D,B,I> original, Cloner cloner)
      : base(original, cloner) {
        PackingItemsRotated = cloner.Clone(original.PackingItemsRotated);
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new RegularSimpleRotationPackingPlan<D,B,I>(this, cloner);
    }

  }
}
